### 离线下载播放
- 支持将课程数据下载到本地，在无网络环境正常播放。

- 进入回放时，需要传入回放的access_token和playbackID，SDK会根据传入的access_token和playbackID从本地检索是否有该课程的缓存文件，如果有缓存则读取本地缓存播放，否则去请求网络。


####下载

##### 初始化
- PlaybackDownloader是一个单例
- PlaybackDownloader初始化整个app生命周期只能初始化一次，一般在自定义Application的onCreate中进行
```
//1.获取单例对象
PlaybackDownloader playbackDownloader = PlaybackDownloader.getInstance();
//2.初始化
playbackDownloader.init(context);
//3.设置下载文件根目录路径，可不设，默认在sd卡创建以app包名的目录为下载根目录
playbackDownloader.setRootFolder(path);
//设置可同时下载的线程数，可不设，默认为3
playbackDownloader.setDownLoadThreadSize(3);
```

#####添加下载任务

| 参数           | 类型                       | 描述                                       |
| ------------ | ------------------------ | ---------------------------------------- |
| access_token | String                   | 回放access_token                           |
| playbackId   | String                   | 回放id                                     |
| title        | String                   | 标题，设置null默认用服务器返回title数据                 |
| thumbnailUrl | String                   | 缩略图地址，设置null默认用服务器返回的缩略图，如果服务器没缩略图，用本地默认图片 |
| listener     | OnappendDownloadListener | 添加下载任务事件回调                               |
```
PlaybackDownloader.getInstance().appendDownloadTask(access_token, playbackId, title,thumbnailUrl, new 	PreDownLoad.OnappendDownloadListener() {
  @Override
  public void success() {
  	....  // 执行下载操作
  }

  @Override
  public void fail(int code, String msg) {
  	.... 
  }
});
```

#####获取下载任务信息
	//获取下载任务ID列表
	List<String> idList = PlaybackDownloader.getInstance().getPlaybackIdList();
	
	获取下载任务信息的两种方式：
	方法一：
	//根据列表位置position获取相应的离线下载任务信息
	DownloadInfoMode info = PlaybackDownloader.getInstance().getDownloadList().get(position);
	方法二：
	//根据回放id获取相应的离线下载任务信息
	DownloadInfoMode info =  PlaybackDownloader.getInstance().getDownloadInfo(id);

#####开始下载离线任务

| 参数         | 类型     | 描述   |
| ---------- | ------ | ---- |
| playbackId | String | 回放id |

	//开始点播id的下载任务
	PlaybackDownloader.getInstance().startDownload(playbackId);

#####下载事件监听

* 通过addDownLoadObserver方法添加相应下载任务状态监听事件

| 参数         | 类型               | 描述         |
| ---------- | ---------------- | ---------- |
| playbackId | String           | 回放id       |
| observer   | DownLoadObserver | 下载任务状态监听事件 |

	//1.实现DownLoadManager.DownLoadObserver接口
	class DownLoadObserverImpl implements DownLoadManager.DownLoadObserver {
	@Override
	public void onDownLoadInfoChange(final DownloadInfoMode info) {
	...
	}
	//2.添加下载事件监听
	DownLoadObserverImpl observer = new DownLoadObserverImpl()
	PlaybackDownloader.getInstance().addDownLoadObserver(playbackId,observer);


#####更新下载状态


	public void onDownLoadInfoChange(final DownloadInfoMode info){
		switch (info.state) {
	    	case PlaybackDownloader.Status.STATE_DOWNLOADED:
	        	//下载完成
	        	...
	        	return;
	    	case PlaybackDownloader.Status.STATE_PAUSEDOWNLOAD:
	        	//停止下载
	        	...
	        	break;
	    	case PlaybackDownloader.Status.STATE_UNDOWNLOAD:
	        	//未下载
	        	...
	        	break;
	    	case PlaybackDownloader.Status.STATE_DOWNLOADING:
	        	//正在下载
	        	...
	        	break;
	    	case PlaybackDownloader.Status.STATE_WAITINGDOWNLOAD:
	        	//等待下载
	        	...
	        	break;
	    	case PlaybackDownloader.Status.STATE_DOWNLOADFAILED:
	        	//下载失败
	        	...
	        	break;
	        }
	}

#####其他下载操作


	//暂停下载某个点播
	PlaybackDownloader.getInstance().pauseDownload(playbackId);
	//暂停全部下载
	PlaybackDownloader.getInstance().pauseAllDownload();
	//删除某个点播任务
	PlaybackDownloader.getInstance().deleteDownload(playbackId)


#####注销

	//destroy()方法会暂停所有正在下载的任务
	PlaybackDownloader.getInstance().destroy();



####播放

​	离线播放不需要做特殊处理，根据在线回放播放逻辑一致，SDK会根据传入的access_token和playbackID从本地检索是否有该课程的缓存文件，如果有缓存则读取本地缓存播放，否则去请求网络。

详情参考:[初始化SDK逻辑（点播）](http://http://open.talk-fun.com/docs/android/SDK_playback_init.html)

